@@ -50,7 +50,7 @@ int main(int argc, char *argv[])
 #endif
     
     /* ltris info */
-    printf( "LTris %s\nCopyright 2002-2020 Michael Speck\nPublished under GNU GPL\n---\n", VERSION );
+    printf( "LTris %s\nCopyright 2002-2023 Michael Speck\nPublished under GNU GPL\n---\n", VERSION );
     printf( "Looking up data in: %s\n", SRC_DIR );
 #ifndef SOUND
     printf( "Compiled without sound and music\n" );
@@ -94,7 +94,14 @@ int main(int argc, char *argv[])
   	}
       }
 
-    init_sdl( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER );
+    init_sdl(SDL_INIT_EVERYTHING);
+    gamepad_open();
+    if (gamepad_opened()) {
+	    printf("  NOTE: Gamepad cannot be configured via menu yet but you\n");
+	    printf("  can edit the gp_ entries in config file %s/%s.\n",
+			    	    config.dir_name, CONFIG_FILE_NAME );
+	    printf("  In case connection to gamepad gets lost, you can press F5 do reconnect.\n");
+    }
 
     info = SDL_GetVideoInfo();
     printf("Display resolution: %d x %d\n", info->current_w, info->current_h);
@@ -157,6 +164,7 @@ int main(int argc, char *argv[])
     chart_delete();
     hint_delete_res();
     
+    gamepad_close();
 #ifdef SOUND
     audio_close();
 #endif
