@@ -29,6 +29,10 @@ extern int  term_game;
 Sdl sdl;
 SDL_Cursor *empty_cursor = 0;
 SDL_Cursor *std_cursor = 0;
+SDL_Joystick *gamepad = 0;
+int gamepad_numbuttons = 0;
+Uint8 gamepad_state[GPAD_LAST1];
+Uint8 gamepad_oldstate[GPAD_LAST1];
 
 /* shadow surface stuff */
 int use_shadow_surface = 0;
@@ -855,9 +859,9 @@ static void select_best_video_mode(int *best_w, int *best_h)
 	int i;
 	int dratio;
 
-	/* with 100 odd resolutions like 1366x768 will be selected
-	 * with 1000 (first decimal must match) these are ruled out */
-	dratio = 1000*display_w/display_h;
+	/* use one decimal place for ratio which allows matching
+	 * slightly odd but roughly matching resolutions like 1366x768 */
+	dratio = 10*display_w/display_h;
 	wanted_mode.w = display_w;
 	wanted_mode.h = display_h;
 
@@ -878,12 +882,12 @@ static void select_best_video_mode(int *best_w, int *best_h)
 			printf("%d x %d, ", modes[i]->w, modes[i]->h);
 		printf("\n");
 
-		/* select lowest mode with same ratio as display */
+		/* select lowest mode >=720p with same ratio as display if any */
 		for(i=0;modes[i];++i)
-			if (1000*modes[i]->w/modes[i]->h == dratio) {
-				if (modes[i]->h < wanted_mode.h)
-					wanted_mode = *modes[i];
-			}
+			if (modes[i]->h >= 720)
+				if (10*modes[i]->w/modes[i]->h == dratio)
+					if (modes[i]->h < wanted_mode.h)
+						wanted_mode = *modes[i];
 		printf("Best mode: %d x %d\n",wanted_mode.w,wanted_mode.h);
 	}
 
@@ -946,7 +950,7 @@ int	set_video_mode( int fullscreen )
 		    video_sh = 480;
 	    }
 	    printf("Using scale factor %s\n",
-			    (video_scale==2)?"2":((video_scale==1)?"1.5":0));
+			    (video_scale==2)?"2":((video_scale==1)?"1.5":"1"));
 	    video_xoff = (w - video_sw) / 2;
 	    video_yoff = (h - video_sh) / 2;
     }
@@ -1396,3 +1400,88 @@ void take_screenshot( int i )
     sprintf( str, "screenshot%i.bmp", i );
     SDL_SaveBMP( sdl.screen, str );
 }
+
+void gamepad_open()
+{
+	gamepad_close();
+
+	/* make sure event queue get button events */
+	SDL_JoystickEventState(SDL_ENABLE);
+
+	if (SDL_NumJoysticks() == 0) {
+		printf("No game controller found...\n");
+		return;
+	}
+
+	if ((gamepad = SDL_JoystickOpen(0)) == NULL) {
+		fprintf(stderr,"Couldn't open game controller: %s\n",SDL_GetError());
+		return;
+	}
+
+	printf("Opened game controller 0\n");
+	printf("  num axes: %d, num buttons: %d, num balls: %d\n",
+			SDL_JoystickNumAxes(gamepad),SDL_JoystickNumButtons(gamepad),
+			SDL_JoystickNumBalls(gamepad));
+
+	gamepad_numbuttons = SDL_JoystickNumButtons(gamepad);
+	if (gamepad_numbuttons > 10)
+		gamepad_numbuttons = 10;
+
+	memset(gamepad_state,0,sizeof(gamepad_state));
+	memset(gamepad_oldstate,0,sizeof(gamepad_oldstate));
+}
+int gamepad_opened()
+{
+	return (gamepad != 0);
+}
+void gamepad_close()
+{
+	if (gamepad) {
+		SDL_JoystickClose(gamepad);
+		gamepad = 0;
+	}
+}
+const Uint8 *gamepad_update()
+{
+	SDL_JoystickUpdate();
+
+	memcpy(gamepad_oldstate,gamepad_state,sizeof(gamepad_state));
+	memset(gamepad_state,0,sizeof(gamepad_state));
+
+	if (gamepad == NULL)
+		return gamepad_state;
+
+	if (SDL_JoystickGetAxis(gamepad,0) < -3200)
+		gamepad_state[GPAD_LEFT] = 1;
+	if (SDL_JoystickGetAxis(gamepad,0) > 3200)
+		gamepad_state[GPAD_RIGHT] = 1;
+	if (SDL_JoystickGetAxis(gamepad,1) < -3200)
+		gamepad_state[GPAD_UP] = 1;
+	if (SDL_JoystickGetAxis(gamepad,1) > 3200)
+		gamepad_state[GPAD_DOWN] = 1;
+	for (uint i = 0; i < gamepad_numbuttons; i++)
+		gamepad_state[GPAD_BUTTON0 + i] = SDL_JoystickGetButton(gamepad,i);
+
+	return gamepad_state;
+}
+
+int gamepad_ctrl_isdown(uint cid)
+{
+	if (cid >= GPAD_LAST1)
+		return 0;
+	if (gamepad_oldstate[cid] == 0 && gamepad_state[cid] == 1)
+		return 1;
+	return 0;
+}
+int gamepad_ctrl_ispressed(uint cid)
+{
+	if (cid >= GPAD_LAST1)
+		return 0;
+	if (gamepad_oldstate[cid] == 1 && gamepad_state[cid] == 1)
+		return 1;
+	return 0;
+}
+int gamepad_ctrl_isactive(uint cid)
+{
+	return gamepad_ctrl_ispressed(cid) || gamepad_ctrl_isdown(cid);
+}
