@@ -23,6 +23,7 @@
 #include "tetris.h"
 
 SDL_Surface *blocks = 0;
+SDL_Surface *previewpieces = 0;
 SDL_Surface *logo = 0;
 Font *font = 0, *large_font = 0;
 SDL_Surface *qmark = 0; /* question mark */
@@ -37,6 +38,7 @@ int  *next_blocks = NULL, next_blocks_size = 0; /* all receive same blocks */
 int last_generated_block = -1; /* block id of last block generated when filling bag */
 
 extern Sdl sdl;
+extern int keystate[SDLK_LAST];
 extern Config config;
 extern int term_game;
 extern char gametype_ids[8][64];
@@ -211,6 +213,29 @@ void tetris_recreate_bkgnd( int id )
     FULL_DEST( sdl.screen ); FULL_SOURCE( offscreen ); blit_surf();
 }
 
+extern Block_Mask block_masks[BLOCK_COUNT];
+
+/** Render preview of all pieces to previewpieces */
+static void render_preview_pieces()
+{
+	int bs = BOWL_BLOCK_SIZE;
+	int xoff[7] = {-10,-10,-10,0,-10,-10,0};
+	int yoff[7] = {-40,-40,-40,-40,-40,-40,-30};
+
+	for (int k = 0; k < 7; k++)
+		for (int j = 0; j < 4; j++)
+			for (int i = 0; i < 4; i++) {
+				if (!block_masks[k].mask[block_masks[k].rstart][i][j])
+					continue;
+	                        DEST( previewpieces,
+	                        	xoff[k] + i*bs,
+	                        	yoff[k] + j*bs + k*bs*2,
+	                        	bs, bs);
+	                        SOURCE( blocks, block_masks[k].blockid * bs, 0);
+	                        blit_surf();
+			}
+}
+
 /*
 ====================================================================
 Publics
@@ -237,6 +262,11 @@ void tetris_create()
     SDL_SetColorKey( offscreen, 0, 0 );
     bowl_load_figures();
     bowl_init_block_masks();
+
+    /* pre-render pieces for preview (7x4x2 in one column) */
+    previewpieces = create_surf(BOWL_BLOCK_SIZE*4,7*BOWL_BLOCK_SIZE*2,SDL_SWSURFACE);
+    render_preview_pieces();
+
 #ifdef SOUND
     wav_click = sound_chunk_load( "click.wav" );
 #endif
@@ -249,6 +279,9 @@ void tetris_delete()
     if ( blocks )
 	    SDL_FreeSurface( blocks );
     blocks = 0;
+    if ( previewpieces )
+	    SDL_FreeSurface( previewpieces );
+    previewpieces = 0;
     if ( qmark )
 	    SDL_FreeSurface( qmark );
     qmark = 0;
@@ -286,46 +319,46 @@ int  tetris_init()
     /* create bowls according to the gametype */
     switch ( config.gametype ) {
         case GAME_DEMO:
-            bowls[0] = bowl_create( 220, 0, 530, 200, blocks, qmark, "Demo", 0 );
+            bowls[0] = bowl_create( 220, 0, 490, 160, -1,-1, blocks, qmark, "Demo", 0 );
             break;
         case GAME_CLASSIC:
         case GAME_TRAINING:
         case GAME_FIGURES:
-            bowls[0] = bowl_create( 220, 0, 530, 200, blocks, qmark, config.player1.name, &config.player1.controls );
+            bowls[0] = bowl_create( 220, 0, 490, config.modern?100:160, 490, 290, blocks, qmark, config.player1.name, &config.player1.controls );
             break;
         case GAME_VS_HUMAN:
         case GAME_VS_CPU:
-            if ( config.center_preview ) {
-                bowls[0] = bowl_create( 20, 0, 273, 200, blocks, qmark, config.player1.name, &config.player1.controls );
-                if ( config.gametype == GAME_VS_HUMAN )
-                    bowls[1] = bowl_create( 420, 0, 367, 200, blocks, qmark, config.player2.name, &config.player2.controls );
-                else
-                    bowls[1] = bowl_create( 420, 0, 367, 200, blocks, qmark, "CPU-1", 0 );
-            }
-            else {
-                bowls[0] = bowl_create( 20, 0, 290, 340, blocks, qmark, config.player1.name, &config.player1.controls );
-                if ( config.gametype == GAME_VS_HUMAN )
-                    bowls[1] = bowl_create( 420, 0, 350, 60, blocks, qmark, config.player2.name, &config.player2.controls );
-                else
-                    bowls[1] = bowl_create( 420, 0, 350, 60, blocks, qmark, "CPU-1", 0 );
-            }
+        	bowls[0] = bowl_create( 20, 0, 233, config.modern?100:160, 233,290,
+        			blocks, qmark, config.player1.name,
+				&config.player1.controls );
+        	if ( config.gametype == GAME_VS_HUMAN )
+        		bowls[1] = bowl_create( 420, 0, 327, config.modern?100:160,
+        				327,290, blocks, qmark, config.player2.name,
+					&config.player2.controls );
+        	else
+        		bowls[1] = bowl_create( 420, 0, 327, config.modern?100:160,
+        				327,290, blocks, qmark, "CPU-1", 0 );
             break;
         case GAME_VS_HUMAN_HUMAN:
         case GAME_VS_HUMAN_CPU:
         case GAME_VS_CPU_CPU:
-            bowls[0] = bowl_create( 10, 0, -1, -1, blocks, qmark, config.player1.name, &config.player1.controls );
+            bowls[0] = bowl_create( 10, 0, -1, -1, -1,-1, blocks, qmark, config.player1.name, &config.player1.controls );
             if ( config.gametype != GAME_VS_CPU_CPU )
-                bowls[1] = bowl_create( 220, 0, -1, -1, blocks, qmark, config.player2.name, &config.player2.controls );
+                bowls[1] = bowl_create( 220, 0, -1, -1, -1,-1, blocks, qmark, config.player2.name, &config.player2.controls );
             else
-                bowls[1] = bowl_create( 220, 0, -1, -1, blocks, qmark, "CPU-1", 0 );
+                bowls[1] = bowl_create( 220, 0, -1, -1, -1,-1, blocks, qmark, "CPU-1", 0 );
             if ( config.gametype == GAME_VS_HUMAN_HUMAN )
-                bowls[2] = bowl_create( 430, 0, -1, -1, blocks, qmark, config.player3.name, &config.player3.controls );
+                bowls[2] = bowl_create( 430, 0, -1, -1, -1,-1, blocks, qmark, config.player3.name, &config.player3.controls );
             else
-                bowls[2] = bowl_create( 430, 0, -1, -1, blocks, qmark, "CPU-2", 0 );
+                bowls[2] = bowl_create( 430, 0, -1, -1, -1,-1, blocks, qmark, "CPU-2", 0 );
             break;
     }
+    /* enable stats for one bowl games */
+    if (config.gametype <= GAME_FIGURES)
+        bowls[0]->show_stats = 1;
     /* background */
-    tetris_recreate_bkgnd( 1 );
+    tetris_recreate_bkgnd(1);
+
     /* shrapnells */
     shrapnells_init();
     return 1;
@@ -367,6 +400,91 @@ void show_fps(int fps) {
 	}
 }
 
+/* Set bow controls by checking pressed/released keys and gamepad */
+void tetris_set_bowl_controls(int i, SDL_Event *ev, BowlControls *bc)
+{
+	Bowl *b = bowls[i];
+	Controls *ctrl = 0;
+
+	memset(bc,0,sizeof(BowlControls));
+
+	/* TODO improve this by using an array in config ... */
+	if (i == 0)
+		ctrl = &config.player1.controls;
+	else if (i == 1)
+		ctrl = &config.player2.controls;
+	else if (i == 2)
+		ctrl = &config.player3.controls;
+	else return;
+
+	if (!b->cpu_player) {
+		if (keystate[ctrl->left])
+			bc->lshift = CS_PRESSED;
+		if (keystate[ctrl->right])
+			bc->rshift = CS_PRESSED;
+		if (keystate[ctrl->down])
+			bc->sdrop = CS_PRESSED;
+
+		if (ev->type == SDL_KEYDOWN) {
+			if (ev->key.keysym.sym == ctrl->left)
+				bc->lshift = CS_DOWN;
+			if (ev->key.keysym.sym == ctrl->right)
+				bc->rshift = CS_DOWN;
+			if (ev->key.keysym.sym == ctrl->rot_left)
+				bc->lrot = CS_DOWN;
+			if (ev->key.keysym.sym == ctrl->rot_right)
+				bc->rrot = CS_DOWN;
+			if (ev->key.keysym.sym == ctrl->drop)
+				bc->hdrop = CS_DOWN;
+			if (ev->key.keysym.sym == ctrl->hold)
+				bc->hold = CS_DOWN;
+		}
+
+		/* allow gamepad for bowl 0 */
+		if (i == 0 && config.gp_enabled) {
+			if (gamepad_ctrl_isdown(GPAD_LEFT))
+				bc->lshift = CS_DOWN;
+			else if (gamepad_ctrl_ispressed(GPAD_LEFT))
+				bc->lshift = CS_PRESSED;
+			if (gamepad_ctrl_isdown(GPAD_RIGHT))
+				bc->rshift = CS_DOWN;
+			else if (gamepad_ctrl_ispressed(GPAD_RIGHT))
+				bc->rshift = CS_PRESSED;
+			if (gamepad_ctrl_isactive(GPAD_DOWN))
+				bc->sdrop = CS_PRESSED;
+
+			if (ev->type == SDL_JOYBUTTONDOWN) {
+				if (ev->jbutton.button == config.gp_lrot)
+					bc->lrot = CS_DOWN;
+				if (ev->jbutton.button == config.gp_rrot)
+					bc->rrot = CS_DOWN;
+				if (ev->jbutton.button == config.gp_hdrop)
+					bc->hdrop = CS_DOWN;
+				if (ev->jbutton.button == config.gp_hold)
+					bc->hold = CS_DOWN;
+			}
+		}
+	}
+
+	/* do cpu move, soft drop is done in bowl_update because of timer */
+	if (b->cpu_player) {
+		/* left/right shift
+		 * XXX we always set PRESSED skipping DOWN ... cheater! */
+		if (b->cpu_dest_x > b->block.x)
+			bc->rshift = CS_PRESSED;
+		else if (b->cpu_dest_x < b->block.x)
+			bc->lshift = CS_PRESSED;
+		/* rotation
+		 * XXX instant, no delay, but CPU is just damn fast isn't it? */
+		if (b->cpu_dest_rot != b->block.rot_id)
+			//if (delay_timed_out( &bowl->cpu_rot_delay, ms))
+			bc->lrot = CS_DOWN;
+		/* if CPU may drop in one p-cycle set key */
+		if (bowl_cpu_may_drop(b))
+			bc->hdrop = CS_DOWN;
+	}
+}
+
 /*
 ====================================================================
 Run a successfully initated game.
@@ -410,22 +528,22 @@ void tetris_run()
     while ( !leave && !term_game ) {
 	    cycle_start = SDL_GetTicks();
 
+	    event.type = SDL_NOEVENT;
         if ( event_poll( &event ) ) {
             switch ( event.type ) {
 		case SDL_QUIT:
 			term_game = 1;
 		    break;
-                case SDL_KEYDOWN:
-                    for ( i = 0; i < BOWL_COUNT; i++ )
-                        if ( bowls[i] ) 
-                            bowl_store_key( bowls[i], event.key.keysym.sym );
-                    break;
                 case SDL_KEYUP:
                     if (game_over)
                         leave = 1;
                     else if (event.key.keysym.sym == config.pause_key)
                         request_pause = 1;
                     else switch ( event.key.keysym.sym ) {
+                        case SDLK_F5:
+                        	gamepad_close();
+                        	gamepad_open();
+                        	break;
                         case SDLK_ESCAPE: 
                             if ( confirm( large_font, _("End Game? y/n"), CONFIRM_YES_NO ) ) 
                                 for ( i = 0; i < BOWL_COUNT; i++ )
@@ -464,9 +582,13 @@ void tetris_run()
             request_pause = 0;
         }
             
+        gamepad_update();
         for ( i = 0; i < BOWL_COUNT; i++ )
-            if ( bowls[i] )
-                bowl_update( bowls[i], ms, game_over );
+            if ( bowls[i] ) {
+                BowlControls bc;
+                tetris_set_bowl_controls(i, &event, &bc);
+                bowl_update( bowls[i], ms, &bc, game_over );
+            }
         
         /* check if any of the bowls entered a new level and change background if so */
         if ( !config.keep_bkgnd ) {
@@ -568,7 +690,7 @@ void tetris_make_stat()
     
     printf( "*****\n" );
     
-    bowl = bowl_create( 0, 0, -1, -1, blocks, qmark, "Demo", 0 );
+    bowl = bowl_create( 0, 0, -1, -1, -1, -1, blocks, qmark, "Demo", 0 );
     
     /* reset counters */
     total = 0; total_lines = 0;
